/***********************************************************
File Name:		isdn_ext.h
Description:	Public variant definition of ISDN module
Version:		v9.0.0
Author:			Gengxin Chen
Create Date:	2008-2-28
************************************************************/
#ifndef _ISDN_EXT_H
#define _ISDN_EXT_H

#include "isdn_if.h"

/************************************************************
ISDN 
************************************************************/


IsdnMsgStr	isdn_sd_msg, isdn_rv_msg;

extern int isdn_init(u8 systemID, u32 interval_ms);
extern void isdn_timer();
/*extern int isdn_bind_sap(ISDN_Sap_Attrib_ *attrib_p);
extern int isdn_modify_sap(u8 sap_id, ISDN_Sap_Attrib_ *attrib_ptr);
extern int isdn_unbind_sap(u8 sap_id);
extern int isdn_add_cg(u8 sap_id, Cg_Attrib_struct *attrib_ptr);
extern int isdn_modify_cg(u16 cg_id, Cg_Attrib_struct *attrib_ptr);
extern int isdn_remove_cg(u16 cg_id);
extern int isdn_add_circuit(u16 cg_id, Circuit_Attrib_struct *attrib_ptr);
extern int isdn_modify_circuit(u16 cg_id, u16 circuit_id, Circuit_Attrib_struct *attrib_ptr);
extern int isdn_remove_circuit(u16 cg_id, u16 circuit_id);*/
extern int isdn_set_run_mode (u8 run_mode, u32 alter_ip);

#endif
